# -*- coding: utf-8 -*-
from collective.volto.formsupport.interfaces import IFormDataStore
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from six import StringIO
from zope.component import getMultiAdapter

import csv
import six


class FormDataExportGet(Service):
    def render(self):
        self.check_permission()

        self.request.response.setHeader(
            "Content-Disposition",
            'attachment; filename="{0}.csv"'.format(self.__name__),
        )
        self.request.response.setHeader(
            "Content-Type", "text/comma-separated-values"
        )

        data = self.get_data()
        if isinstance(data, six.text_type):
            data = data.encode("utf-8")
        self.request.response.write(data)

    def get_data(self):
        store = getMultiAdapter((self.context, self.request), IFormDataStore)
        sbuf = StringIO()
        fixed_columns = ["date"]
        columns = []

        rows = []
        for item in store.search():
            data = {}
            for k, v in item.attrs.items():
                if k == "block_id":
                    continue
                if k not in columns and k not in fixed_columns:
                    columns.append(k)
                data[k] = json_compatible(v)
            rows.append(data)
        columns.extend(fixed_columns)
        writer = csv.DictWriter(sbuf, fieldnames=columns, delimiter=",")
        writer.writeheader()
        for row in rows:
            writer.writerow(row)
        res = sbuf.getvalue()
        sbuf.close()
        return res



# -*- coding: utf-8 -*-
from collective.volto.formsupport.interfaces import IFormDataStore
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.services import Service
from zope.component import getMultiAdapter
from zope.interface import alsoProvides


class FormDataClear(Service):
    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        store = getMultiAdapter((self.context, self.request), IFormDataStore)
        store.clear()

        return self.reply_no_content()



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
    quickInstallProduct,
)
from plone.restapi.testing import PloneRestApiDXLayer
from plone.testing import z2

import collective.MockMailHost
import collective.volto.formsupport
import plone.restapi


class VoltoFormsupportLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.restapi

        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=collective.volto.formsupport)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "plone.restapi:blocks")
        applyProfile(portal, "collective.volto.formsupport:default")


VOLTO_FORMSUPPORT_FIXTURE = VoltoFormsupportLayer()


VOLTO_FORMSUPPORT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(VOLTO_FORMSUPPORT_FIXTURE,),
    name="VoltoFormsupportLayer:IntegrationTesting",
)


VOLTO_FORMSUPPORT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(VOLTO_FORMSUPPORT_FIXTURE,),
    name="VoltoFormsupportLayer:FunctionalTesting",
)


class VoltoFormsupportRestApiLayer(PloneRestApiDXLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        super(VoltoFormsupportRestApiLayer, self).setUpZope(
            app, configurationContext
        )
        self.loadZCML(package=collective.MockMailHost)
        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=collective.volto.formsupport)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "collective.volto.formsupport:default")
        applyProfile(portal, "plone.restapi:blocks")
        quickInstallProduct(portal, "collective.MockMailHost")
        applyProfile(portal, "collective.MockMailHost:default")


VOLTO_FORMSUPPORT_API_FIXTURE = VoltoFormsupportRestApiLayer()
VOLTO_FORMSUPPORT_API_INTEGRATION_TESTING = IntegrationTesting(
    bases=(VOLTO_FORMSUPPORT_API_FIXTURE,),
    name="VoltoFormsupportRestApiLayer:Integration",
)

VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(VOLTO_FORMSUPPORT_API_FIXTURE, z2.ZSERVER_FIXTURE),
    name="VoltoFormsupportRestApiLayer:Functional",
)



# -*- coding: utf-8 -*-
from collective.volto.formsupport.testing import (
    VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING,  # noqa: E501,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestBlockSerialization(unittest.TestCase):

    layer = VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.document = api.content.create(
            type="Document", title="Example context", container=self.portal,
        )
        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {
                "@type": "form",
                "default_from": "foo@foo.com",
                "default_bar": "bar",
                "name": {},
                "surname": {},
            },
        }
        self.document_url = self.document.absolute_url()
        api.content.transition(obj=self.document, transition="publish")
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_serializer_return_full_block_data_to_admin(self):
        response = self.api_session.get(self.document_url)
        res = response.json()
        self.assertEqual(
            res["blocks"]["form-id"], self.document.blocks["form-id"]
        )

    def test_serializer_return_filtered_block_data_to_anon(self):
        response = self.anon_api_session.get(self.document_url)
        res = response.json()
        self.assertNotEqual(
            res["blocks"]["form-id"], self.document.blocks["form-id"]
        )
        self.assertNotIn("default_from", res["blocks"]["form-id"].keys())
        self.assertNotIn("default_foo", res["blocks"]["form-id"].keys())
        self.assertIn("name", res["blocks"]["form-id"].keys())
        self.assertIn("surname", res["blocks"]["form-id"].keys())



# -*- coding: utf-8 -*-
from email.message import EmailMessage
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from plone.registry.interfaces import IRegistry
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from collective.volto.formsupport import _
from zope.i18n import translate
from collective.volto.formsupport.interfaces import IFormDataStore

import codecs
import six


class SubmitPost(Service):
    def __init__(self, context, request):
        super(SubmitPost, self).__init__(context, request)

        self.block = {}
        self.form_data = json_body(self.request)
        self.block_id = self.form_data.get("block_id", "")
        if self.block_id:
            self.block = self.get_block_data(block_id=self.block_id)

    def reply(self):
        self.validate_form()

        store_action = self.block.get("store", False)
        send_action = self.block.get("send", False)

        # Disable CSRF protection
        alsoProvides(self.request, IDisableCSRFProtection)

        if store_action:
            self.store_data()
        if send_action:
            self.send_data()

        return self.reply_no_content()

    def validate_form(self):
        """
        check all required fields and parameters
        """
        if not self.block_id:
            raise BadRequest(
                translate(
                    _("missing_blockid_label", default="Missing block_id"),
                    context=self.request,
                )
            )
        if not self.block:
            raise BadRequest(
                translate(
                    _(
                        "block_form_not_found_label",
                        default='Block with @type "form" and id "$block" not found in this context: $context',
                        mapping={
                            "block": self.block_id,
                            "context": self.context.absolute_url(),
                        },
                    ),
                    context=self.request,
                ),
            )

        if not self.block.get("store", False) and not self.block.get("send", False):
            raise BadRequest(
                translate(
                    _(
                        "missing_action",
                        default='You need to set at least one form action between "send" and "store".',  # noqa
                    ),
                    context=self.request,
                )
            )

        if not self.form_data.get("data", []):
            raise BadRequest(
                translate(
                    _(
                        "empty_form_data",
                        default="Empty form data.",
                    ),
                    context=self.request,
                )
            )

    def get_block_data(self, block_id):
        blocks = getattr(self.context, "blocks", {})
        if not blocks:
            return {}
        for id, block in blocks.items():
            if id != block_id:
                continue
            block_type = block.get("@type", "")
            if block_type != "form":
                continue
            return block
        return {}

    def get_reply_to(self):
        """This method retrieves the correct field to be used as 'reply to'.

        Three "levels" of logic:
        1. If there is a field marked with 'use_as_reply_to' set to True, that
           field wins and we use that.
           If not:
        2. We search for the "from" field.
           If not present:
        3. We use the fallback field: "default_from"
        """

        subblocks = self.block.get("subblocks", "")
        if subblocks:
            for field in subblocks:
                if field.get("use_as_reply_to", False):
                    field_id = field.get("field_id", "")
                    if field_id:
                        for data in self.form_data.get("data", ""):
                            if data.get("field_id", "") == field_id:
                                return data["value"]

        return self.form_data.get("from", "") or self.block.get("default_from", "")

    def send_data(self):
        subject = self.form_data.get("subject", "") or self.block.get(
            "default_subject", ""
        )

        mfrom = self.form_data.get("from", "") or self.block.get("default_from", "")
        mreply_to = self.get_reply_to()

        if not subject or not mfrom:
            raise BadRequest(
                translate(
                    _(
                        "send_required_field_missing",
                        default="Missing required field: subject or from.",
                    ),
                    context=self.request,
                )
            )

        portal = api.portal.get()
        overview_controlpanel = getMultiAdapter(
            (portal, self.request), name="overview-controlpanel"
        )
        if overview_controlpanel.mailhost_warning():
            raise BadRequest("MailHost is not configured.")

        registry = getUtility(IRegistry)
        mail_settings = registry.forInterface(IMailSchema, prefix="plone")
        mto = self.block.get("default_to", mail_settings.email_from_address)
        encoding = registry.get("plone.email_charset", "utf-8")
        message = self.prepare_message()

        msg = EmailMessage()
        msg.set_content(message)
        msg["Subject"] = subject
        msg["From"] = mfrom
        msg["To"] = mto
        msg["Reply-To"] = mreply_to
        msg.replace_header("Content-Type", 'text/html; charset="utf-8"')

        self.manage_attachments(msg=msg)

        self.send_mail(msg=msg, encoding=encoding)

    def prepare_message(self):
        message_template = api.content.get_view(
            name="send_mail_template",
            context=self.context,
            request=self.request,
        )
        parameters = {
            "parameters": self.filter_parameters(),
            "url": self.context.absolute_url(),
            "title": self.context.Title(),
        }
        return message_template(**parameters)

    def filter_parameters(self):
        """
        do not send attachments fields.
        """
        skip_fields = [
            x.get("field_id", "")
            for x in self.block.get("subblocks", [])
            if x.get("field_type", "") == "attachment"
        ]
        return [
            x
            for x in self.form_data.get("data", [])
            if x.get("field_id", "") not in skip_fields
        ]

    def send_mail(self, msg, encoding):
        host = api.portal.get_tool(name="MailHost")
        # try:

        host.send(msg, charset=encoding)

        # except (SMTPException, RuntimeError):
        #     plone_utils = api.portal.get_tool(name="plone_utils")
        #     exception = plone_utils.exceptionString()
        #     message = "Unable to send mail: {}".format(exception)

        #     self.request.response.setStatus(500)
        #     return dict(
        #         error=dict(type="InternalServerError", message=message)
        #     )

    def manage_attachments(self, msg):
        attachments = self.form_data.get("attachments", {})
        if not attachments:
            return []
        for key, value in attachments.items():
            content_type = "application/octet-stream"
            filename = None
            if isinstance(value, dict):
                file_data = value.get("data", "")
                if not file_data:
                    continue
                content_type = value.get("content-type", content_type)
                filename = value.get("filename", filename)
                if isinstance(file_data, six.text_type):
                    file_data = file_data.encode("utf-8")
                if "encoding" in value:
                    file_data = codecs.decode(file_data, value["encoding"])
                if isinstance(file_data, six.text_type):
                    file_data = file_data.encode("utf-8")
            else:
                file_data = value
            msg.add_attachment(
                file_data,
                maintype=content_type,
                subtype=content_type,
                filename=filename,
            )

    def store_data(self):
        store = getMultiAdapter((self.context, self.request), IFormDataStore)
        res = store.add(data=self.filter_parameters())
        if not res:
            raise BadRequest("Unable to store data")



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.volto.formsupport.testing import (
    VOLTO_FORMSUPPORT_INTEGRATION_TESTING,  # noqa: E501,
)
from plone import api
from plone.app.testing import setRoles, TEST_USER_ID

import unittest


try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


class TestSetup(unittest.TestCase):
    """Test that collective.volto.formsupport is properly installed."""

    layer = VOLTO_FORMSUPPORT_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")

    def test_product_installed(self):
        """Test if collective.volto.formsupport is installed."""
        self.assertTrue(
            self.installer.isProductInstalled("collective.volto.formsupport")
        )

    def test_browserlayer(self):
        """Test that ICollectiveVoltoFormsupportLayer is registered."""
        from collective.volto.formsupport.interfaces import (
            ICollectiveVoltoFormsupportLayer,
        )
        from plone.browserlayer import utils

        self.assertIn(
            ICollectiveVoltoFormsupportLayer, utils.registered_layers()
        )


class TestUninstall(unittest.TestCase):

    layer = VOLTO_FORMSUPPORT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer.uninstallProducts(["collective.volto.formsupport"])
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if collective.volto.formsupport is cleanly uninstalled."""
        self.assertFalse(
            self.installer.isProductInstalled("collective.volto.formsupport")
        )

    def test_browserlayer_removed(self):
        """Test that ICollectiveVoltoFormsupportLayer is removed."""
        from collective.volto.formsupport.interfaces import (
            ICollectiveVoltoFormsupportLayer,
        )
        from plone.browserlayer import utils

        self.assertNotIn(
            ICollectiveVoltoFormsupportLayer, utils.registered_layers()
        )



# -*- coding: utf-8 -*-
from collective.volto.formsupport.interfaces import (
    ICollectiveVoltoFormsupportLayer,
)
from plone.restapi.behaviors import IBlocks
from plone.restapi.interfaces import IBlockFieldSerializationTransformer
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapter
from zope.interface import implementer
from plone import api


class FormSerializer(object):
    """
    """

    order = 200  # after standard ones
    block_type = "form"

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, value):
        """
        If user can edit the context, return the full block data.
        Otherwise, skip default values because we need them only in edit and
        to send emails from the backend.
        """
        if api.user.has_permission(
            "Modify portal content", obj=self.context
        ):
            return value
        return {k: v for k, v in value.items() if not k.startswith("default_")}


@implementer(IBlockFieldSerializationTransformer)
@adapter(IBlocks, ICollectiveVoltoFormsupportLayer)
class FormSerializerContents(FormSerializer):
    """ Deserializer for content-types that implements IBlocks behavior """


@implementer(IBlockFieldSerializationTransformer)
@adapter(IPloneSiteRoot, ICollectiveVoltoFormsupportLayer)
class FormSerializerRoot(FormSerializer):
    """ Deserializer for site-root """



# -*- coding: utf-8 -*-
from collective.volto.formsupport.interfaces import IFormDataStore
from plone import api
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

import json
import six


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class FormData(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        if not self.show_component():
            return {}

        result = {
            "form_data": {"@id": "{}/@form-data".format(self.context.absolute_url())}
        }
        if not expand:
            return result

        store = getMultiAdapter((self.context, self.request), IFormDataStore)
        data = store.search()
        items = [self.expand_records(x) for x in data]
        data = {
            "@id": "{}/@form-data".format(self.context.absolute_url()),
            "items": items,
            "items_total": len(items),
        }

        result["form_data"] = data
        return result

    def show_component(self):
        if not api.user.has_permission("Modify portal content", obj=self.context):
            return False
        blocks = getattr(self.context, "blocks", {})
        if isinstance(blocks, six.text_type):
            blocks = json.loads(blocks)
        if not blocks:
            return False
        for block in blocks.values():
            if block.get("@type", "") == "form" and block.get("store", False):
                return True
        return False

    def expand_records(self, record):
        data = {k: json_compatible(v) for k, v in record.attrs.items()}
        data["id"] = record.intid
        return data


class FormDataGet(Service):
    def reply(self):
        form_data = FormData(self.context, self.request)
        return form_data(expand=True).get("form_data", {})



# -*- coding: utf-8 -*-
from collective.volto.formsupport.testing import (
    VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING,  # noqa: E501,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from plone.restapi.testing import RelativeSession
from Products.MailHost.interfaces import IMailHost
from zope.component import getUtility

import transaction
import unittest


class TestMailSend(unittest.TestCase):

    layer = VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.mailhost = getUtility(IMailHost)

        registry = getUtility(IRegistry)
        registry["plone.email_from_address"] = "site_addr@plone.com"
        registry["plone.email_from_name"] = u"Plone test site"

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.document = api.content.create(
            type="Document",
            title="Example context",
            container=self.portal,
        )
        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form"},
        }
        self.document_url = self.document.absolute_url()
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

        # set default block
        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form"},
        }
        transaction.commit()

    def submit_form(self, data):
        url = "{}/@submit-form".format(self.document_url)
        response = self.api_session.post(
            url,
            json=data,
        )
        transaction.commit()
        return response

    def test_email_not_send_if_block_id_is_not_given(self):
        response = self.submit_form(
            data={"from": "john@doe.com", "message": "Just want to say hi."},
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(res["message"], "Missing block_id")

    def test_email_not_send_if_block_id_is_incorrect_or_not_present(self):
        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "block_id": "unknown",
            },
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            'Block with @type "form" and id "unknown" not found in this context: {}'.format(  # noqa
                self.document_url
            ),
        )

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "message": "Just want to say hi.",
                "block_id": "text-id",
            },
        )
        transaction.commit()

        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            'Block with @type "form" and id "text-id" not found in this context: {}'.format(  # noqa
                self.document_url
            ),
        )

    def test_email_not_send_if_no_action_set(self):

        response = self.submit_form(
            data={"from": "john@doe.com", "block_id": "form-id"},
        )
        transaction.commit()
        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            'You need to set at least one form action between "send" and "store".',  # noqa
        )

    def test_email_not_send_if_block_id_is_correct_but_form_data_missing(
        self,
    ):

        self.document.blocks = {
            "form-id": {"@type": "form", "send": True},
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            "Empty form data.",
        )

    def test_email_not_send_if_block_id_is_correct_but_required_fields_missing(
        self,
    ):

        self.document.blocks = {
            "form-id": {"@type": "form", "send": True},
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "block_id": "form-id",
                "data": [{"label": "foo", "value": "bar"}],
            },
        )
        transaction.commit()
        res = response.json()
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            res["message"],
            "Missing required field: subject or from.",
        )

    def test_email_sent_with_site_recipient(
        self,
    ):

        self.document.blocks = {
            "form-id": {"@type": "form", "send": True},
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"label": "Message", "value": "just want to say hi"},
                    {"label": "Name", "value": "John"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: test subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: site_addr@plone.com", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("<strong>Message:</strong> just want to say hi", msg)
        self.assertIn("<strong>Name:</strong> John", msg)

    def test_email_sent_ignore_passed_recipient(
        self,
    ):

        self.document.blocks = {
            "form-id": {"@type": "form", "send": True},
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "to": "to@spam.com",
                "data": [
                    {"label": "Message", "value": "just want to say hi"},
                    {"label": "Name", "value": "John"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: test subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: site_addr@plone.com", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("<strong>Message:</strong> just want to say hi", msg)
        self.assertIn("<strong>Name:</strong> John", msg)

    def test_email_sent_with_block_recipient_if_set(
        self,
    ):

        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {
                "@type": "form",
                "default_to": "to@block.com",
                "send": True,
            },
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"label": "Message", "value": "just want to say hi"},
                    {"label": "Name", "value": "John"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: test subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: to@block.com", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("<strong>Message:</strong> just want to say hi", msg)
        self.assertIn("<strong>Name:</strong> John", msg)

    def test_email_sent_with_block_subject_if_set_and_not_passed(
        self,
    ):

        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {
                "@type": "form",
                "default_subject": "block subject",
                "send": True,
            },
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"label": "Message", "value": "just want to say hi"},
                    {"label": "Name", "value": "John"},
                ],
                "block_id": "form-id",
            },
        )
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: block subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: site_addr@plone.com", msg)
        self.assertIn("Reply-To: john@doe.com", msg)
        self.assertIn("<strong>Message:</strong> just want to say hi", msg)
        self.assertIn("<strong>Name:</strong> John", msg)

    def test_email_with_use_as_reply_to(
        self,
    ):

        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {
                "@type": "form",
                "default_subject": "block subject",
                "default_from": "john@doe.com",
                "send": True,
                "subblocks": [
                    {
                        "field_id": "contact",
                        "field_type": "from",
                        "use_as_reply_to": True,
                    },
                ],
            },
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "data": [
                    {"label": "Message", "value": "just want to say hi"},
                    {"label": "Name", "value": "Smith"},
                    {"field_id": "contact", "label": "Email", "value": "smith@doe.com"},
                ],
                "block_id": "form-id",
            },
        )
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        msg = self.mailhost.messages[0]
        if isinstance(msg, bytes) and bytes is not str:
            # Python 3 with Products.MailHost 4.10+
            msg = msg.decode("utf-8")
        self.assertIn("Subject: block subject", msg)
        self.assertIn("From: john@doe.com", msg)
        self.assertIn("To: site_addr@plone.com", msg)
        self.assertIn("Reply-To: smith@doe.com", msg)
        self.assertIn("<strong>Message:</strong> just want to say hi", msg)
        self.assertIn("<strong>Name:</strong> Smith", msg)



# -*- coding: utf-8 -*-
from collective.volto.formsupport.interfaces import IFormDataStore
from copy import deepcopy
from datetime import datetime
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.deserializer import json_body
from repoze.catalog.catalog import Catalog
from repoze.catalog.indexes.field import CatalogFieldIndex
from souper.interfaces import ICatalogFactory
from souper.soup import get_soup
from souper.soup import NodeAttributeIndexer
from souper.soup import Record
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.component import getUtility
from plone.i18n.normalizer.interfaces import IIDNormalizer


import logging

logger = logging.getLogger(__name__)


@implementer(ICatalogFactory)
class FormDataSoupCatalogFactory(object):
    def __call__(self, context):
        #  do not set any index here..maybe on each form
        catalog = Catalog()
        block_id_indexer = NodeAttributeIndexer("block_id")
        catalog[u"block_id"] = CatalogFieldIndex(block_id_indexer)
        return catalog


@implementer(IFormDataStore)
@adapter(IDexterityContent, Interface)
class FormDataStore(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def soup(self):
        return get_soup("form_data", self.context)

    @property
    def block_id(self):
        data = json_body(self.request)
        if not data:
            data = self.request.form
        return data.get("block_id", "")

    def get_form_fields(self):
        blocks = getattr(self.context, "blocks", {})
        if not blocks:
            return {}
        form_block = {}
        for id, block in blocks.items():
            if id != self.block_id:
                continue
            block_type = block.get("@type", "")
            if block_type == "form":
                form_block = deepcopy(block)
        if not form_block:
            return {}
        return form_block.get("subblocks", [])

    def add(self, data):
        form_fields = self.get_form_fields()
        if not form_fields:
            logger.error(
                'Block with id {} and type "form" not found in context: {}.'.format(
                    self.block_id, self.context.absolute_url()
                )
            )
            return None

        form_ids = [x.get("field_id", "") for x in form_fields]
        record = Record()
        # record.attrs["metadata"] = {}
        normalizer = getUtility(IIDNormalizer)
        for field in data:
            field_id = field.get("field_id", "")
            id = normalizer.normalize(field.get("label", ""))
            value = field.get("value", "")
            if field_id in form_ids:
                record.attrs[id] = value
                # record.attrs["metadata"][id] = {
                #     "field_id": field_id,
                #     "label": field.get("label", ""),
                # }
        record.attrs["date"] = datetime.now()
        record.attrs["block_id"] = self.block_id
        return self.soup.add(record)

    def length(self):
        return len([x for x in self.soup.data.values()])

    def search(self, query=None):
        if not query:
            records = sorted(
                self.soup.data.values(),
                key=lambda k: k.attrs.get("date", ""),
                reverse=True,
            )
        return records

    def delete(self, id):
        record = self.soup.get(id)
        del self.soup[record]

    def clear(self):
        self.soup.clear()



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from copy import deepcopy
from plone import api
from plone.dexterity.utils import iterSchemata
from zope.schema import getFields

try:
    from collective.volto.blocksfield.field import BlocksField

    HAS_BLOCKSFIELD = True
except ImportError:
    HAS_BLOCKSFIELD = False

import logging
import json

logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-collective.volto.formsupport:default"


def to_1100(context):  # noqa: C901
    logger.info("### START CONVERSION FORM BLOCKS ###")

    def fix_block(blocks, url):
        for block in blocks.values():
            if block.get("@type", "") != "form":
                continue
            found = False
            for field in block.get("subblocks", []):
                if field.get("field_type", "") == "checkbox":
                    field["field_type"] = "multiple_choice"
                    found = True
                if field.get("field_type", "") == "radio":
                    field["field_type"] = "simple_choice"
                    found = True
            if found:
                logger.info("[CONVERTED] - {}".format(url))

    # fix root
    portal = api.portal.get()
    portal_blocks = getattr(portal, "blocks", "")
    if portal_blocks:
        blocks = json.loads(portal_blocks)
        fix_block(blocks, portal.absolute_url())
        portal.blocks = json.dumps(blocks)

    # fix blocks in contents
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = aq_base(brain.getObject())
        for schema in iterSchemata(item):
            for name, field in getFields(schema).items():
                if name == "blocks":
                    blocks = deepcopy(item.blocks)
                    if blocks:
                        fix_block(blocks, brain.getURL())
                        item.blocks = blocks
                elif HAS_BLOCKSFIELD and isinstance(field, BlocksField):
                    value = deepcopy(field.get(item))
                    if not value:
                        continue
                    if isinstance(value, str):
                        if value == "":
                            setattr(
                                item,
                                name,
                                {"blocks": {}, "blocks_layout": {"items": []}},
                            )
                            continue
                    if blocks:
                        fix_block(blocks, brain.getURL())
                        setattr(item, name, value)



# -*- coding: utf-8 -*-
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.interface import Interface


class ICollectiveVoltoFormsupportLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IFormDataStore(Interface):
    def add(self, data):
        """
        Add data to the store

        @return: record id
        """

    def length(self):
        """
        @return: number of items stored into store
        """

    def search(self, query):
        """
        @return: items that match query
        """
